@@ -183,7 +183,7 @@ show_book_error (GNCBackendError io_error, const char *newfile)
       break;
 
     case ERR_SQL_DB_BUSY:
-      fmt = _("The SQL database is in use by other users,"
+      fmt = _("The SQL database is in use by other users, "
               "and the upgrade cannot be performed until they logoff.\n"
               "If there are currently no other users, consult the \n"
               "documentation to learn how to clear out dangling login\n"
@@ -275,7 +275,7 @@ gncFileNew (void)
 gboolean
 gncFileQuerySave (void)
 {
-  GNCBook * book = gncGetCurrentBook();
+  GNCBook * book;
 
   /* If user wants to mess around before finishing business with
    * the old file, give em a chance to figure out what's up.  
@@ -283,7 +283,7 @@ gncFileQuerySave (void)
    * up the file-selection dialog, we don't blow em out of the water;
    * instead, give them another chance to say "no" to the verify box.
    */
-  while (gnc_book_not_saved(book)) {
+  while (gnc_book_not_saved(gncGetCurrentBook())) {
     GNCVerifyResult result;
     const char *message = _("Changes have been made since the last "
                             "Save. Save the data to file?");
