@@ -1762,6 +1762,40 @@ xaccAccountsHaveCommonCurrency(Account *account_1, Account *account_2)
 /********************************************************************\
 \********************************************************************/
 
+gboolean
+xaccAccountGetPlaceholder (Account *account)
+{
+  kvp_value *kvp;
+  char *setting;
+
+  if ( ( account )                                      &&
+       ( kvp = kvp_frame_get_slot (account->kvp_data, "placeholder" ) ) &&
+       ( kvp_value_get_type (kvp) == KVP_TYPE_STRING ) && 
+       ( setting = kvp_value_get_string(kvp) ) &&
+       ( !strcmp( setting, "true" ) ) )
+      return TRUE;
+
+  return FALSE;
+}
+
+void
+xaccAccountSetPlaceholder (Account *account, gboolean option)
+{
+  if (!account)
+    return;
+
+  xaccAccountBeginEdit (account);
+  kvp_frame_set_slot_nc(account->kvp_data, "placeholder",
+			kvp_value_new_string (option ? "true" : "false"));
+
+  mark_account (account);
+  account->core_dirty = TRUE;
+  xaccAccountCommitEdit (account);
+}
+
+/********************************************************************\
+\********************************************************************/
+
 gboolean
 xaccAccountHasAncestor (Account *account, Account * ancestor)
 {
@@ -2226,6 +2260,49 @@ xaccAccountGetQuoteTZ(Account *acc)
 /********************************************************************\
 \********************************************************************/
 
+void
+xaccAccountSetReconcileChildrenStatus(Account *account, gboolean status)
+{ 
+  kvp_frame *frame;
+  if (!account)
+    return;
+  
+  xaccAccountBeginEdit (account);
+  
+  frame = kvp_frame_get_frame (account->kvp_data, "reconcile-info", NULL);
+  kvp_frame_set_slot_nc (frame,
+			 "include-children",
+                         status ? kvp_value_new_gint64 (status) : NULL);
+  account->core_dirty = TRUE;
+  xaccAccountCommitEdit (account);
+  return;
+}
+
+/********************************************************************\
+\********************************************************************/
+
+gboolean
+xaccAccountGetReconcileChildrenStatus(Account *account)
+{
+  kvp_value *status;
+  if (!account)
+    return FALSE;
+  /* access the account's kvp-data for status and return that, if no value
+   * is found then we can assume not to include the children, that being
+   * the default behaviour 
+   */
+  status = kvp_frame_get_slot_path (account->kvp_data,
+				    "reconcile-info",
+				    "include-children",
+				    NULL);
+  if (!status)
+    return FALSE;
+  return kvp_value_get_gint64 (status);
+}
+
+/********************************************************************\
+\********************************************************************/
+
 gboolean
 xaccAccountVisitUnvisitedTransactions(Account *acc,
                                       gboolean (*proc)(Transaction *t,
