@@ -558,6 +558,8 @@ gnc_get_source_name (PriceSourceCode source)
       return "TIAA-CREF";
     case SOURCE_TRUSTNET :
       return "Trustnet";
+    case SOURCE_VWD :
+      return "VWD";
     default:
       break;
   }
@@ -590,6 +592,8 @@ gnc_get_source_code_name (PriceSourceCode source)
       return "TIAACREF";
     case SOURCE_TRUSTNET :
       return "TRUSTNET";
+    case SOURCE_VWD :
+      return "VWD";
     default:
       break;
   }
@@ -624,7 +628,7 @@ equity_base_name (GNCEquityType equity_type)
   switch (equity_type)
   {
     case EQUITY_OPENING_BALANCE:
-      return N_("Opening Balances");
+      return N_("Opening Balance");
 
     case EQUITY_RETAINED_EARNINGS:
       return N_("Retained Earnings");
