@@ -26,7 +26,7 @@
 
 #include <gtk/gtkctree.h>
 
-#include "dialog-utils.h"
+#include "gnc-ui-util.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -55,23 +55,26 @@ struct _GNCAccountTree
 {
   GtkCTree ctree;
 
-  AccountFilter filter;
-  gpointer filter_data;
+  AccountFilter view_filter;
+  gpointer view_filter_data;
+
+  AccountFilter selectable_filter;
+  gpointer selectable_filter_data;
 
   AccountViewInfo avi;
 
-  gint    num_columns;
-  gint    balance_column;
-  gint    total_column;
-  gint    column_fields[NUM_ACCOUNT_FIELDS];
+  gint num_columns;
+  gint balance_column;
+  gint total_column;
+  AccountFieldCode column_fields[NUM_ACCOUNT_FIELDS];
 
   const gchar * column_headings[NUM_ACCOUNT_FIELDS + 1];
 
   GtkStyle *deficit_style;
 
-  Account *root_account;
+  GUID root_account;
 
-  GList   *current_accounts;
+  GList * current_accounts;
 
   gboolean ignore_unselect;
 };
@@ -102,18 +105,25 @@ GtkWidget * gnc_account_tree_new_with_root (Account *account);
 
 void gnc_account_tree_refresh (GNCAccountTree *tree);
 
-void gnc_account_tree_refresh_all (void);
-
-void gnc_account_tree_set_view_info(GNCAccountTree *tree,
-				    AccountViewInfo *info);
-
 gboolean gnc_account_tree_select_account (GNCAccountTree *tree,
 					  Account *account,
                                           gboolean show_account);
 
-gboolean gnc_account_tree_select_accounts(GNCAccountTree *tree,
-                                          GList          *account_list,
-                                          gboolean        show_last);
+gboolean gnc_account_tree_select_subaccounts (GNCAccountTree *tree,
+                                              Account *account,
+                                              gboolean show_account);
+
+gboolean gnc_account_tree_select_accounts (GNCAccountTree *tree,
+                                           GList          *account_list,
+                                           gboolean        show_last);
+
+gboolean gnc_account_tree_unselect_account (GNCAccountTree *tree,
+                                            Account        *account,
+                                            gboolean       show_account);
+
+gboolean gnc_account_tree_unselect_subaccounts (GNCAccountTree *tree,
+                                                Account *account,
+                                                gboolean show_account);
 
 void gnc_account_tree_expand_account (GNCAccountTree *tree,
                                       Account *account);
@@ -121,21 +131,20 @@ void gnc_account_tree_expand_account (GNCAccountTree *tree,
 void gnc_account_tree_toggle_account_expansion (GNCAccountTree *tree,
                                                 Account *account);
 
-void gnc_account_tree_insert_account (GNCAccountTree *tree,
-				      Account *account);
+void gnc_account_tree_expand_all (GNCAccountTree *tree);
 
-void gnc_account_tree_remove_account (GNCAccountTree *tree,
-				      Account *account);
+void gnc_account_tree_show_income_expense (GNCAccountTree *tree);
 
-void gnc_account_tree_remove_account_all(Account *account);
-
-void gnc_account_tree_show_categories (GNCAccountTree *tree);
-
-void gnc_account_tree_hide_categories (GNCAccountTree *tree);
+void gnc_account_tree_hide_income_expense (GNCAccountTree *tree);
 
 Account * gnc_account_tree_get_current_account (GNCAccountTree *tree);
 GList *   gnc_account_tree_get_current_accounts (GNCAccountTree *tree);
 
+Account * gnc_account_tree_get_focus_account (GNCAccountTree *tree);
+
+gboolean  gnc_account_tree_account_selected (GNCAccountTree *tree,
+                                             Account *account);
+
 void gnc_account_tree_hide_all_but_name (GNCAccountTree *tree);
 
 void gnc_init_account_view_info(AccountViewInfo *avi);
@@ -146,9 +155,13 @@ void gnc_account_tree_set_view_info (GNCAccountTree *tree,
 void gnc_account_tree_get_view_info (GNCAccountTree *tree,
 				     AccountViewInfo *info);
 
-void gnc_account_tree_set_filter (GNCAccountTree *tree,
-                                  AccountFilter filter,
-                                  gpointer user_data);
+void gnc_account_tree_set_view_filter (GNCAccountTree *tree,
+                                       AccountFilter filter,
+                                       gpointer user_data);
+
+void gnc_account_tree_set_selectable_filter (GNCAccountTree *tree,
+                                             AccountFilter filter,
+                                             gpointer user_data);
 
 #ifdef __cplusplus
 }
