@@ -115,6 +115,9 @@ fill_account_list (StockSplitInfo *info, Account *account)
     if (gnc_numeric_zero_p (balance))
       continue;
 
+    if (xaccAccountGetPlaceholder (account))
+	continue;
+
     security = xaccAccountGetSecurity (account);
 
     print_info = gnc_account_quantity_print_info (account, FALSE);
