@@ -1,4 +1,6 @@
 /********************************************************************\
+ * combocell-gnome.c -- implement combobox pull down cell for gnome *
+ *                                                                  *
  * This program is free software; you can redistribute it and/or    *
  * modify it under the terms of the GNU General Public License as   *
  * published by the Free Software Foundation; either version 2 of   *
@@ -34,16 +36,15 @@
 
 #include <gnome.h>
 
-#include "splitreg.h"
-#include "table-allgui.h"
-#include "table-gnome.h"
 #include "combocell.h"
-#include "gnucash-sheet.h"
+#include "gnc-engine-util.h"
+#include "gnc-ui-util.h"
 #include "gnucash-item-edit.h"
 #include "gnucash-item-list.h"
-#include "global-options.h"
+#include "gnucash-sheet.h"
 #include "messages.h"
-#include "util.h"
+#include "splitreg.h"
+#include "table-allgui.h"
 
 
 typedef struct _PopBox
@@ -60,56 +61,63 @@ typedef struct _PopBox
         gboolean list_sorted;  /* list has been sorted? */
         gboolean list_popped;  /* list is popped up? */
 
+        gboolean autosize;
+
         QuickFill *qf;
         gboolean in_list_select;
 
-        gncBoolean strict;
+        gboolean strict;
 
         char complete_char; /* char to be used for auto-completion */
 
-        gchar *ignore_string;
-        gchar *ignore_help;
+        GList *ignore_strings;
+        GList *ignore_helps;
 } PopBox;
 
 
 static void block_list_signals (ComboCell *cell);
 static void unblock_list_signals (ComboCell *cell);
-static void realizeCombo (BasicCell *bcell, void *w, int width);
-static void moveCombo (BasicCell *bcell, int phys_row, int phys_col);
+static void realizeCombo (BasicCell *bcell, gpointer w);
+static void moveCombo (BasicCell *bcell, VirtualLocation virt_loc);
 static void destroyCombo (BasicCell *bcell);
-static const char * enterCombo (BasicCell *bcell,
-                                const char *value,
-                                int *cursor_position,
-                                int *start_selection,
-                                int *end_selection);
-static const char * leaveCombo (BasicCell *bcell, const char *value);
+static gboolean enterCombo (BasicCell *bcell,
+                            int *cursor_position,
+                            int *start_selection,
+                            int *end_selection);
+static void leaveCombo (BasicCell *bcell);
 
 /* This static indicates the debugging module that this .o belongs to.  */
 static short module = MOD_GTK_REG;
+static gboolean auto_pop_combos = FALSE;
+
 
 /* =============================================== */
 
-ComboCell *xaccMallocComboCell (void)
+ComboCell *
+xaccMallocComboCell (void)
 {
-	ComboCell * cell = (ComboCell *) malloc (sizeof (ComboCell));
+	ComboCell * cell;
 
-	assert(cell != NULL);
+        cell = g_new0 (ComboCell, 1);
 
-	xaccInitComboCell(cell);
+	xaccInitComboCell (cell);
 
 	return cell;
 }
 
-void xaccInitComboCell (ComboCell *cell)
+void
+xaccInitComboCell (ComboCell *cell)
 {
 	PopBox *box;
 
-	xaccInitBasicCell(&(cell->cell));
+	xaccInitBasicCell (&(cell->cell));
+
+        cell->cell.is_popup = TRUE;
 
 	cell->cell.realize = realizeCombo;
 	cell->cell.destroy = destroyCombo;
 
-	box = g_new0(PopBox, 1);
+	box = g_new0 (PopBox, 1);
 
 	box->sheet = NULL;
 	box->item_edit = NULL;
@@ -119,18 +127,19 @@ void xaccInitComboCell (ComboCell *cell)
 	box->list_in_sync = TRUE;
         box->list_sorted = TRUE;
         box->list_popped = FALSE;
+        box->autosize = FALSE;
 
 	cell->cell.gui_private = box;
 
-        box->qf = xaccMallocQuickFill();
+        box->qf = gnc_quickfill_new ();
         box->in_list_select = FALSE;
 
         box->strict = TRUE;
 
-        box->complete_char = 0;
+        box->complete_char = '\0';
 
-        box->ignore_string = NULL;
-        box->ignore_help = NULL;
+        box->ignore_strings = NULL;
+        box->ignore_helps = NULL;
 }
 
 /* =============================================== */
@@ -138,52 +147,54 @@ void xaccInitComboCell (ComboCell *cell)
 static void
 select_item_cb (GNCItemList *item_list, char *item_string, gpointer data)
 {
-	ComboCell *cell = (ComboCell *) data;
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	ComboCell *cell = data;
+	PopBox *box = cell->cell.gui_private;
 
         box->in_list_select = TRUE;
-	gnucash_sheet_modify_current_cell(box->sheet, item_string);
+	gnucash_sheet_modify_current_cell (box->sheet, item_string);
         box->in_list_select = FALSE;
 
-        item_edit_hide_list(box->item_edit);
+        item_edit_hide_popup (box->item_edit);
         box->list_popped = FALSE;
 }
 
 static void
 change_item_cb (GNCItemList *item_list, char *item_string, gpointer data)
 {
-	ComboCell *cell = (ComboCell *) data;
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	ComboCell *cell = data;
+	PopBox *box = cell->cell.gui_private;
 
         box->in_list_select = TRUE;
-	gnucash_sheet_modify_current_cell(box->sheet, item_string);
+	gnucash_sheet_modify_current_cell (box->sheet, item_string);
         box->in_list_select = FALSE;
 }
 
 static void
 activate_item_cb (GNCItemList *item_list, char *item_string, gpointer data)
 {
-	ComboCell *cell = (ComboCell *) data;
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	ComboCell *cell = data;
+	PopBox *box = cell->cell.gui_private;
 
-        item_edit_hide_list(box->item_edit);
+        item_edit_hide_popup (box->item_edit);
         box->list_popped = FALSE;
 }
 
 static void
 key_press_item_cb (GNCItemList *item_list, GdkEventKey *event, gpointer data)
 {
-	ComboCell *cell = (ComboCell *) data;
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	ComboCell *cell = data;
+	PopBox *box = cell->cell.gui_private;
 
-        switch(event->keyval) {
+        switch (event->keyval)
+        {
                 case GDK_Escape:
-                        item_edit_hide_list (box->item_edit);
+                        item_edit_hide_popup (box->item_edit);
                         box->list_popped = FALSE;
                         break;
+
                 default:
-                        gtk_widget_event(GTK_WIDGET(box->sheet),
-                                         (GdkEvent *) event);
+                        gtk_widget_event (GTK_WIDGET(box->sheet),
+                                          (GdkEvent *) event);
                         break;
         }
 }
@@ -191,12 +202,12 @@ key_press_item_cb (GNCItemList *item_list, GdkEventKey *event, gpointer data)
 static void
 combo_disconnect_signals (ComboCell *cell)
 {
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	PopBox *box = cell->cell.gui_private;
 
 	if (!box->signals_connected)
 		return;
 
-        if (GTK_OBJECT_DESTROYED(GTK_OBJECT(box->item_list)))
+        if (GTK_OBJECT_DESTROYED (GTK_OBJECT(box->item_list)))
                 return;
 
 	gtk_signal_disconnect_by_data (GTK_OBJECT(box->item_list), cell);
@@ -207,7 +218,7 @@ combo_disconnect_signals (ComboCell *cell)
 static void
 combo_connect_signals (ComboCell *cell)
 {
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	PopBox *box = cell->cell.gui_private;
 
 	if (box->signals_connected)
 		return;
@@ -215,17 +226,17 @@ combo_connect_signals (ComboCell *cell)
         if (GTK_OBJECT_DESTROYED(GTK_OBJECT(box->item_list)))
                 return;
 
-        gtk_signal_connect(GTK_OBJECT(box->item_list), "select_item",
-                           GTK_SIGNAL_FUNC(select_item_cb), cell);
+        gtk_signal_connect (GTK_OBJECT(box->item_list), "select_item",
+                            GTK_SIGNAL_FUNC(select_item_cb), cell);
 
-        gtk_signal_connect(GTK_OBJECT(box->item_list), "change_item",
-                           GTK_SIGNAL_FUNC(change_item_cb), cell);
+        gtk_signal_connect (GTK_OBJECT(box->item_list), "change_item",
+                            GTK_SIGNAL_FUNC(change_item_cb), cell);
 
-        gtk_signal_connect(GTK_OBJECT(box->item_list), "activate_item",
-                           GTK_SIGNAL_FUNC(activate_item_cb), cell);
+        gtk_signal_connect (GTK_OBJECT(box->item_list), "activate_item",
+                            GTK_SIGNAL_FUNC(activate_item_cb), cell);
 
-        gtk_signal_connect(GTK_OBJECT(box->item_list), "key_press_event",
-                           GTK_SIGNAL_FUNC(key_press_item_cb), cell);
+        gtk_signal_connect (GTK_OBJECT(box->item_list), "key_press_event",
+                            GTK_SIGNAL_FUNC(key_press_item_cb), cell);
 
 	box->signals_connected = TRUE;
 }
@@ -233,7 +244,7 @@ combo_connect_signals (ComboCell *cell)
 static void
 block_list_signals (ComboCell *cell)
 {
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	PopBox *box = cell->cell.gui_private;
 
 	if (!box->signals_connected)
 		return;
@@ -244,7 +255,7 @@ block_list_signals (ComboCell *cell)
 static void
 unblock_list_signals (ComboCell *cell)
 {
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	PopBox *box = cell->cell.gui_private;
 
 	if (!box->signals_connected)
 		return;
@@ -257,12 +268,13 @@ unblock_list_signals (ComboCell *cell)
 static void
 destroyCombo (BasicCell *bcell)
 {
-	PopBox *box = (PopBox *) bcell->gui_private;
+	PopBox *box = bcell->gui_private;
 	ComboCell *cell = (ComboCell *) bcell;
 
 	if (cell->cell.realize == NULL)
 	{
-		if (box != NULL && box->item_list != NULL) {
+		if (box != NULL && box->item_list != NULL)
+                {
 			combo_disconnect_signals(cell);
 			gtk_object_unref(GTK_OBJECT(box->item_list));
 			box->item_list = NULL;
@@ -282,29 +294,48 @@ destroyCombo (BasicCell *bcell)
 /* =============================================== */
 
 static void
-menustring_free(gpointer string, gpointer user_data)
+menustring_free (gpointer string, gpointer user_data)
 {
-        g_free(string);
+        g_free (string);
 }
 
 /* =============================================== */
 
-void xaccDestroyComboCell (ComboCell *cell)
+void
+xaccDestroyComboCell (ComboCell *cell)
 {
-	PopBox *box = (PopBox *) cell->cell.gui_private;
+	PopBox *box = cell->cell.gui_private;
 
 	destroyCombo(&(cell->cell));
 
-	if (box != NULL) {
+	if (box != NULL)
+        {
+                GList *node;
+
 		g_list_foreach(box->menustrings, menustring_free, NULL);
 		g_list_free(box->menustrings);
                 box->menustrings = NULL;
 
-                xaccFreeQuickFill(box->qf);
+                gnc_quickfill_destroy (box->qf);
                 box->qf = NULL;
 
-                g_free(box->ignore_string);
-                g_free(box->ignore_help);
+                for (node = box->ignore_strings; node; node = node->next)
+                {
+                        g_free (node->data);
+                        node->data = NULL;
+                }
+
+                g_list_free (box->ignore_strings);
+                box->ignore_strings = NULL;
+
+                for (node = box->ignore_helps; node; node = node->next)
+                {
+                        g_free (node->data);
+                        node->data = NULL;
+                }
+
+                g_list_free (box->ignore_helps);
+                box->ignore_helps = NULL;
 
 		g_free(box);
 		cell->cell.gui_private = NULL;
@@ -326,7 +357,7 @@ xaccClearComboCellMenu (ComboCell * cell)
         if (cell == NULL)
                 return;
 
-        box = (PopBox *) cell->cell.gui_private;
+        box = cell->cell.gui_private;
         if (box == NULL)
                 return;
         if (box->menustrings == NULL)
@@ -336,8 +367,8 @@ xaccClearComboCellMenu (ComboCell * cell)
         g_list_free(box->menustrings);
         box->menustrings = NULL;
 
-        xaccFreeQuickFill(box->qf);
-        box->qf = xaccMallocQuickFill();
+        gnc_quickfill_destroy (box->qf);
+        box->qf = gnc_quickfill_new ();
 
         if (box->item_list != NULL)
         {
@@ -357,10 +388,10 @@ xaccClearComboCellMenu (ComboCell * cell)
 static void
 gnc_append_string_to_list(gpointer _string, gpointer _item_list)
 {
-	char *string = (char *) _string;
-	GNCItemList *item_list = GNC_ITEM_LIST(_item_list);
+	char *string = _string;
+	GNCItemList *item_list = GNC_ITEM_LIST (_item_list);
 
-	gnc_item_list_append(item_list, string);
+	gnc_item_list_append (item_list, string);
 }
 
 static void
@@ -369,9 +400,9 @@ gnc_combo_sync_edit_list(PopBox *box)
 	if (box->list_in_sync || box->item_list == NULL)
 		return;
 
-	gnc_item_list_clear(box->item_list);
-	g_list_foreach(box->menustrings, gnc_append_string_to_list,
-		       box->item_list);
+	gnc_item_list_clear (box->item_list);
+	g_list_foreach (box->menustrings, gnc_append_string_to_list,
+                        box->item_list);
 }
 
 static void
@@ -393,25 +424,26 @@ xaccAddComboCellMenuItem (ComboCell *cell, char * menustr)
 	if (menustr == NULL)
 		return;
 
-	box = (PopBox *) cell->cell.gui_private;
-	box->menustrings = g_list_append(box->menustrings, g_strdup(menustr));
+	box = cell->cell.gui_private;
+	box->menustrings = g_list_append (box->menustrings,
+                                          g_strdup (menustr));
 
 	gnc_combo_sync_edit_list(box);
 
 	if (box->item_list != NULL)
         {
-                block_list_signals(cell);
+                block_list_signals (cell);
 
-		gnc_item_list_append(box->item_list, menustr);
-                if (safe_strcmp(menustr, cell->cell.value) == 0)
-                        gnc_item_list_select(box->item_list, menustr);
+		gnc_item_list_append (box->item_list, menustr);
+                if (safe_strcmp (menustr, cell->cell.value) == 0)
+                        gnc_item_list_select (box->item_list, menustr);
 
-                unblock_list_signals(cell);
+                unblock_list_signals (cell);
         }
 	else
 		box->list_in_sync = FALSE;
 
-        xaccQFInsertText(box->qf, menustr, QUICKFILL_ALPHA);
+        gnc_quickfill_insert (box->qf, menustr, QUICKFILL_ALPHA);
 
         box->list_sorted = FALSE;
 }
@@ -421,98 +453,93 @@ xaccAddComboCellMenuItem (ComboCell *cell, char * menustr)
 void
 xaccSetComboCellValue (ComboCell *cell, const char *str)
 {
-	xaccSetBasicCellValue(&cell->cell, str);
+	xaccSetBasicCellValue (&cell->cell, str);
 }
 
 /* =============================================== */
 
-static const char *
+static void
 ComboMV (BasicCell *_cell,
-         const char *oldval,
-         const char *change,
-	 const char *newval,
+         const GdkWChar *change,
+         int change_len,
+	 const GdkWChar *newval,
+         int newval_len,
          int *cursor_position,
          int *start_selection,
          int *end_selection)
 {
         ComboCell *cell = (ComboCell *) _cell;
 	PopBox *box = cell->cell.gui_private;
-        const char *retval;
+        const char *match_str;
         QuickFill *match;
         gboolean pop_list;
 
         if (box->in_list_select)
         {
-                xaccSetBasicCellValue (_cell, newval);
+                xaccSetBasicCellWCValueInternal (_cell, newval);
 
                 *cursor_position = -1;
                 *start_selection = 0;
                 *end_selection = -1;
 
-                return newval;
+                return;
         }
 
         /* If deleting, just accept */
         if (change == NULL)
         {
-                xaccSetBasicCellValue (_cell, newval);
-                return newval;
+                xaccSetBasicCellWCValueInternal (_cell, newval);
+                return;
         }
 
         /* If we are inserting in the middle, just accept */
-        if (*cursor_position < strlen(oldval))
+        if (*cursor_position < _cell->value_len)
         {
-                xaccSetBasicCellValue (_cell, newval);
-                return newval;
+                xaccSetBasicCellWCValueInternal (_cell, newval);
+                return;
         }
 
-        match = xaccGetQuickFillStr(box->qf, newval);
+        match = gnc_quickfill_get_string_match (box->qf, newval);
 
-        if ((match == NULL) || (match->text == NULL))
+        match_str = gnc_quickfill_string (match);
+
+        if ((match == NULL) || (match_str == NULL))
         {
-                xaccSetBasicCellValue (_cell, newval);
+                xaccSetBasicCellWCValueInternal (_cell, newval);
 
-                block_list_signals(cell);
-                gnc_item_list_select(box->item_list, NULL);
-                unblock_list_signals(cell);
+                block_list_signals (cell);
+                gnc_item_list_select (box->item_list, NULL);
+                unblock_list_signals (cell);
 
-                return newval;
+                return;
         }
 
-        retval = strdup(match->text);
-
-        *start_selection = strlen(newval);
+        *start_selection = newval_len;
         *end_selection = -1;
-        *cursor_position += strlen(change);
+        *cursor_position += change_len;
 
         if (!box->list_popped)
-                pop_list = gnc_lookup_boolean_option("Register",
-                                                     "Auto-Raise Lists",
-                                                     TRUE);
+                pop_list = auto_pop_combos;
         else
                 pop_list = FALSE;
 
         if (pop_list)
         {
-                item_edit_show_list(box->item_edit);
+                item_edit_show_popup (box->item_edit);
                 box->list_popped = TRUE;
         }
 
-        block_list_signals(cell);
-	gnc_item_list_select(box->item_list, retval);
-        unblock_list_signals(cell);
+        block_list_signals (cell);
+	gnc_item_list_select (box->item_list, match_str);
+        unblock_list_signals (cell);
 
-        xaccSetBasicCellValue (_cell, retval);
-
-        return retval;
+        xaccSetBasicCellValueInternal (_cell, match_str);
 }
 
 /* =============================================== */
 
-static gncBoolean
+static gboolean
 ComboDirect (BasicCell *bcell,
-             const char *oldval,
-             char **newval_ptr,
              int *cursor_position,
              int *start_selection,
              int *end_selection,
@@ -524,15 +551,13 @@ ComboDirect (BasicCell *bcell,
         gboolean keep_on_going = FALSE;
         gboolean extra_colon;
         QuickFill *match;
-        char *search;
+        const char *match_str;
         int prefix_len;
+        int find_pos;
         int new_pos;
-        int length;
 
         if (event->type != GDK_KEY_PRESS)
-                return GNC_F;
-
-        length = strlen(oldval);
+                return FALSE;
 
         switch (event->keyval) {
                 case GDK_slash:
@@ -541,138 +566,159 @@ ComboDirect (BasicCell *bcell,
                                 if (event->keyval == box->complete_char)
                                         break;
 
-                                return GNC_F;
+                                return FALSE;
                         }
                         keep_on_going = TRUE;
+                        /* Fall through */
                 case GDK_Tab:
                 case GDK_ISO_Left_Tab:
                         if (!(event->state & GDK_CONTROL_MASK) &&
                             !keep_on_going)
-                                return GNC_F;
+                                return FALSE;
 
-                        match = xaccGetQuickFillStrLen(box->qf, oldval,
-                                                       *cursor_position);
+                        match = gnc_quickfill_get_string_len_match
+                                (box->qf, bcell->value_w, *cursor_position);
                         if (match == NULL)
-                                return GNC_T;
+                                return TRUE;
 
-                        match = xaccGetQuickFillUniqueLen(match, &prefix_len);
+                        match = gnc_quickfill_get_unique_len_match
+                                (match, &prefix_len);
                         if (match == NULL)
-                                return GNC_T;
+                                return TRUE;
 
-                        if ((match->text != NULL) &&
-                            (strncmp(match->text, oldval, length) == 0) && 
-                            (strcmp(match->text, oldval) != 0))
+                        match_str = gnc_quickfill_string (match);
+
+                        if ((match_str != NULL) &&
+                            (strncmp (match_str, bcell->value,
+                                      strlen (bcell->value)) == 0) && 
+                            (strcmp (match_str, bcell->value) != 0))
                         {
-                                *newval_ptr = strdup(match->text);
-                                assert(*newval_ptr != NULL);
-
-                                xaccSetBasicCellValue(bcell, *newval_ptr);
-
-                                block_list_signals(cell);
-                                gnc_item_list_select(box->item_list,
-                                                     *newval_ptr);
-                                unblock_list_signals(cell);
+                                xaccSetBasicCellValueInternal (bcell,
+                                                               match_str);
 
+                                block_list_signals (cell);
+                                gnc_item_list_select (box->item_list,
+                                                      match_str);
+                                unblock_list_signals (cell);
                         }
 
                         *cursor_position += prefix_len;
                         *start_selection = *cursor_position;
                         *end_selection = -1;
 
-                        return GNC_T;
+                        return TRUE;
         }
 
         if (box->complete_char == 0)
-                return GNC_F;
+                return FALSE;
 
         if (event->keyval != box->complete_char)
-                return GNC_F;
+                return FALSE;
 
         if (event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
-                return GNC_F;
+                return FALSE;
 
-        if ((*cursor_position < length) &&
-            ((*end_selection < length) ||
+        if ((*cursor_position < bcell->value_len) &&
+            ((*end_selection < bcell->value_len) ||
              (*cursor_position < *start_selection)))
-                return GNC_F;
+                return FALSE;
 
-        if ((*cursor_position == length) &&
+        if ((*cursor_position == bcell->value_len) &&
             (*start_selection != *end_selection) &&
-            (*end_selection < length))
-                return GNC_F;
+            (*end_selection < bcell->value_len))
+                return FALSE;
 
-        search = NULL;
-        if (*cursor_position < length)
-                search = strchr(oldval + *cursor_position + 1,
-                                box->complete_char);
+        find_pos = -1;
+        if (*cursor_position < bcell->value_len)
+        {
+                int i = *cursor_position;
+
+                while (bcell->value_w[i])
+                {
+                        if (bcell->value_w[i++] == box->complete_char)
+                        {
+                                find_pos = i;
+                                break;
+                        }
+                }
+        }
 
         new_pos = *cursor_position;
 
-        if (search != NULL)
+        if (find_pos >= 0)
         {
-                new_pos = search - oldval;
+                new_pos = find_pos;
                 extra_colon = FALSE;
         }
         else
         {
-                new_pos = length;
+                new_pos = bcell->value_len;
                 extra_colon = TRUE;
         }
 
-        match = xaccGetQuickFillStrLen(box->qf, oldval, new_pos);
+        match = gnc_quickfill_get_string_len_match (box->qf,
+                                                    bcell->value_w, new_pos);
         if (match == NULL)
-                return GNC_F;
+                return FALSE;
 
         if (extra_colon)
         {
-                match = xaccGetQuickFill(match, box->complete_char);
+                match = gnc_quickfill_get_char_match (match,
+                                                      box->complete_char);
                 if (match == NULL)
-                        return GNC_F;
+                        return FALSE;
 
                 new_pos++;
         }
 
-        if ((match->text != NULL) &&
-            (strncmp(match->text, oldval, length) == 0) && 
-            (strcmp(match->text, oldval) != 0))
+        match_str = gnc_quickfill_string (match);
+
+        if ((match_str != NULL) &&
+            (strncmp (match_str, bcell->value, strlen (bcell->value)) == 0) && 
+            (strcmp (match_str, bcell->value) != 0))
         {
-                *newval_ptr = strdup(match->text);
-                assert(*newval_ptr != NULL);
+                xaccSetBasicCellValueInternal (bcell, match_str);
 
-                xaccSetBasicCellValue(bcell, *newval_ptr);
-
-                block_list_signals(cell);
-                gnc_item_list_select(box->item_list, *newval_ptr);
-                unblock_list_signals(cell);
+                block_list_signals (cell);
+                gnc_item_list_select (box->item_list, match_str);
+                unblock_list_signals (cell);
         }
 
         *cursor_position = new_pos;
         *start_selection = new_pos;
         *end_selection = -1;
 
-        return GNC_T;
+        return TRUE;
 }
 
 /* =============================================== */
 
 static char *
-ComboHelpValue(BasicCell *bcell)
+ComboHelpValue (BasicCell *bcell)
 {
         ComboCell *cell = (ComboCell *) bcell;
         PopBox *box = cell->cell.gui_private;
 
         if ((bcell->value != NULL) && (bcell->value[0] != 0))
         {
-                if ((box->ignore_string != NULL) &&
-                    (box->ignore_help != NULL) &&
-                    (safe_strcmp(bcell->value, box->ignore_string) == 0))
-                        return strdup(box->ignore_help);
+                GList *node;
+                GList *help_node = box->ignore_helps;
 
-                return strdup(bcell->value);
+                for (node = box->ignore_strings; node;
+                     node = node->next, help_node = help_node->next)
+                {
+                        if (safe_strcmp (bcell->value, node->data) != 0)
+                                continue;
+
+                        if (help_node->data != NULL)
+                                return g_strdup (help_node->data);
+                }
+
+                return g_strdup (bcell->value);
         }
 
         if (bcell->blank_help != NULL)
-                return strdup(bcell->blank_help);
+                return g_strdup (bcell->blank_help);
 
         return NULL;
 }
@@ -680,11 +726,11 @@ ComboHelpValue(BasicCell *bcell)
 /* =============================================== */
 
 static void
-realizeCombo (BasicCell *bcell, void *data, int pixel_width)
+realizeCombo (BasicCell *bcell, gpointer data)
 {
-	GnucashSheet *sheet = (GnucashSheet *) data;
+	GnucashSheet *sheet = data;
 	GnomeCanvasItem *item = sheet->item_editor;
-	ItemEdit *item_edit = ITEM_EDIT(item);
+	ItemEdit *item_edit = ITEM_EDIT (item);
 	ComboCell *cell = (ComboCell *) bcell;
 	PopBox *box = cell->cell.gui_private;
 
@@ -692,8 +738,8 @@ realizeCombo (BasicCell *bcell, void *data, int pixel_width)
 	box->sheet = sheet;
 	box->item_edit = item_edit;
 	box->item_list = item_edit_new_list(box->item_edit);
-	gtk_object_ref(GTK_OBJECT(box->item_list));
-	gtk_object_sink(GTK_OBJECT(box->item_list));
+	gtk_object_ref (GTK_OBJECT(box->item_list));
+	gtk_object_sink (GTK_OBJECT(box->item_list));
 
 	/* to mark cell as realized, remove the realize method */
 	cell->cell.realize = NULL;
@@ -709,105 +755,154 @@ realizeCombo (BasicCell *bcell, void *data, int pixel_width)
 /* =============================================== */
 
 static void
-moveCombo (BasicCell *bcell, int phys_row, int phys_col)
+moveCombo (BasicCell *bcell, VirtualLocation virt_loc)
 {
-	PopBox *box = (PopBox *) bcell->gui_private;
+	PopBox *box = bcell->gui_private;
 
-	combo_disconnect_signals((ComboCell *) bcell);
+	combo_disconnect_signals ((ComboCell *) bcell);
 
-	gnome_canvas_item_set(GNOME_CANVAS_ITEM(box->item_edit),
-			      "is_combo", FALSE, NULL);
-
-	item_edit_set_list(box->item_edit, NULL);
+	item_edit_set_popup (box->item_edit, NULL, NULL,
+                             NULL, NULL, NULL, NULL, NULL);
 
         box->list_popped = FALSE;
 }
 
 /* =============================================== */
 
-static const char *
+static int
+get_popup_height (GnomeCanvasItem *item,
+                  int space_available,
+                  int row_height,
+                  gpointer user_data)
+{
+        return space_available;
+}
+
+static int
+popup_autosize (GnomeCanvasItem *item,
+                int max_width,
+                gpointer user_data)
+{
+        PopBox *box = user_data;
+
+        if (!box || !box->autosize)
+                return max_width;
+
+        return gnc_item_list_autosize (GNC_ITEM_LIST (item)) + 20;
+}
+
+static void
+popup_set_focus (GnomeCanvasItem *item,
+                 gpointer user_data)
+{
+        gtk_widget_grab_focus (GTK_WIDGET (GNC_ITEM_LIST (item)->clist));
+}
+
+static void
+popup_post_show (GnomeCanvasItem *item,
+                 gpointer user_data)
+{
+        /* What the hell is this doing here? Well, under gtk+ 1.2.9,
+         * the scrollbars never appear without it. Why does it work?
+         * Why are you asking so many questions? There's nothing to
+         * see here. These aren't the droids you're looking for.
+         * Move along. */
+        gtk_widget_size_request (GNC_ITEM_LIST (item)->frame, NULL);
+
+        gnc_item_list_autosize (GNC_ITEM_LIST (item));
+        gnc_item_list_show_selected (GNC_ITEM_LIST (item));
+}
+
+static int
+popup_get_width (GnomeCanvasItem *item,
+                 gpointer user_data)
+{
+        return GTK_WIDGET (GNC_ITEM_LIST (item)->clist)->allocation.width;
+}
+
+static gboolean
 enterCombo (BasicCell *bcell,
-            const char *value,
             int *cursor_position,
             int *start_selection,
             int *end_selection)
 {
         ComboCell *cell = (ComboCell *) bcell;
-	PopBox *box = (PopBox *) bcell->gui_private;
+	PopBox *box = bcell->gui_private;
+        GList *find;
 
-        if ((box->ignore_string != NULL) &&
-            (safe_strcmp(value, box->ignore_string) == 0))
-                return strdup(value);
+        find = g_list_find_custom (box->ignore_strings,
+                                   bcell->value,
+                                   (GCompareFunc) safe_strcmp);
+        if (find)
+                return FALSE;
 
-	gnc_combo_sync_edit_list(box);
-        gnc_combo_sort_edit_list(box);
+	gnc_combo_sync_edit_list (box);
+        gnc_combo_sort_edit_list (box);
 
-	item_edit_set_list(box->item_edit, box->item_list);
+	item_edit_set_popup (box->item_edit,
+                             GNOME_CANVAS_ITEM (box->item_list),
+                             get_popup_height, popup_autosize,
+                             popup_set_focus, popup_post_show,
+                             popup_get_width, box);
 
-	gnome_canvas_item_set(GNOME_CANVAS_ITEM(box->item_edit),
-			      "is_combo", TRUE, NULL);
+        block_list_signals (cell);
+	gnc_item_list_select (box->item_list, bcell->value);
+        unblock_list_signals (cell);
 
-        block_list_signals(cell);
-	gnc_item_list_select(box->item_list, bcell->value);
-        unblock_list_signals(cell);
-
-	combo_connect_signals((ComboCell *) bcell);
+	combo_connect_signals (cell);
 
         *cursor_position = -1;
         *start_selection = 0;
         *end_selection = -1;
 
-	return NULL;
+	return TRUE;
 }
 
 /* =============================================== */
 
-static const char *
-leaveCombo (BasicCell *bcell, const char *value)
+static void
+leaveCombo (BasicCell *bcell)
 {
-        GList *find;
+	PopBox *box = bcell->gui_private;
 
-	PopBox *box = (PopBox *) bcell->gui_private;
+	combo_disconnect_signals ((ComboCell *) bcell);
 
-	combo_disconnect_signals((ComboCell *) bcell);
-
-	gnome_canvas_item_set(GNOME_CANVAS_ITEM(box->item_edit),
-			      "is_combo", FALSE, NULL);
-
-	item_edit_set_list(box->item_edit, NULL);
+	item_edit_set_popup (box->item_edit, NULL, NULL,
+                             NULL, NULL, NULL, NULL, NULL);
 
         box->list_popped = FALSE;
 
         if (box->strict)
         {
-                find = g_list_find_custom(box->menustrings,
-                                          (gpointer) value,
-                                          (GCompareFunc) safe_strcmp);
+                GList *find;
 
-                /* The ignore string is ok, even if it's not in list. */
-                if (find == NULL &&
-                    ((box->ignore_string == NULL) ||
-                    (safe_strcmp(value, box->ignore_string) != 0)))
-                {
-                        xaccSetBasicCellValue(bcell, "");
-                        return strdup("");
-                }
+                find = g_list_find_custom (box->menustrings,
+                                           bcell->value,
+                                           (GCompareFunc) safe_strcmp);
+                if (find)
+                        return;
+
+                find = g_list_find_custom (box->ignore_strings,
+                                           bcell->value,
+                                           (GCompareFunc) safe_strcmp);
+                if (find)
+                        return;
+
+                xaccSetBasicCellValueInternal (bcell, "");
         }
-
-	return value;
 }
 
 /* =============================================== */
 
 void
-xaccComboCellSetStrict (ComboCell *cell, gncBoolean strict)
+xaccComboCellSetStrict (ComboCell *cell, gboolean strict)
 {
 	PopBox *box;
 
 	if (cell == NULL)
 		return;
 
-	box = (PopBox *) cell->cell.gui_private;
+	box = cell->cell.gui_private;
 
         box->strict = strict;
 }
@@ -822,7 +917,7 @@ xaccComboCellSetCompleteChar (ComboCell *cell, char complete_char)
 	if (cell == NULL)
 		return;
 
-	box = (PopBox *) cell->cell.gui_private;
+	box = cell->cell.gui_private;
 
         box->complete_char = complete_char;
 }
@@ -830,31 +925,46 @@ xaccComboCellSetCompleteChar (ComboCell *cell, char complete_char)
 /* =============================================== */
 
 void
-xaccComboCellSetIgnoreString (ComboCell *cell, const char *ignore_string)
+xaccComboCellAddIgnoreString (ComboCell *cell,
+                              const char *ignore_string,
+                              const char *ignore_help)
 {
 	PopBox *box;
 
 	if (cell == NULL)
 		return;
 
-	box = (PopBox *) cell->cell.gui_private;
+	box = cell->cell.gui_private;
 
-        box->ignore_string = g_strdup(ignore_string);
+        box->ignore_strings = g_list_prepend (box->ignore_strings,
+                                              g_strdup (ignore_string));
+        box->ignore_helps = g_list_prepend (box->ignore_helps,
+                                            g_strdup (ignore_help));
 }
 
 /* =============================================== */
 
 void
-xaccComboCellSetIgnoreHelp (ComboCell *cell, const char *ignore_help)
+xaccComboCellSetAutoSize (ComboCell *cell, gboolean autosize)
 {
 	PopBox *box;
 
-	if (cell == NULL)
-		return;
+        if (!cell)
+                return;
 
-	box = (PopBox *) cell->cell.gui_private;
+	box = cell->cell.gui_private;
+        if (!box)
+                return;
 
-        box->ignore_help = g_strdup(ignore_help);
+        box->autosize = autosize;
+}
+
+/* =============================================== */
+
+void
+xaccComboCellSetAutoPop (gboolean auto_pop_combos_arg)
+{
+        auto_pop_combos = auto_pop_combos_arg;
 }
 
 /* =============== end of file =================== */
