@@ -37,6 +37,8 @@
 ;;
 
 (use-modules (www main))
+(use-modules (srfi srfi-1))
+
 ;; (use-modules (srfi srfi-19)) when available (see below).
 
 (define (yahoo-get-historical-quotes symbol
@@ -352,6 +354,8 @@
      ((string=? "VANGUARD" src) 'vanguard)
      ((string=? "ASX" src) 'asx)
      ((string=? "TIAACREF" src) 'tiaacref)
+     ((string=? "TRUSTNET" src) 'trustnet)
+     ((string=? "VWD" src) 'vwd)
      ((string=? "CURRENCY" src) 'currency)
      (else #f)))
 
@@ -365,6 +369,8 @@
      ((vanguard) "vanguard")
      ((asx) "asx")
      ((tiaacref) "tiaacref")
+     ((trustnet) "trustnet")
+     ((vwd) "vwd")
      ((currency) "currency")
      (else #f)))
 
@@ -382,20 +388,31 @@
        currency
        fq-method-sym
        mnemonic
-       (cons fq-method-sym (list commodity currency tz)))))
+       (list fq-method-sym commodity currency tz))))
 
-  (let ((cmd-list (delete #f (map account->fq-cmd account-list)))
-	(cmd-hash (make-hash-table 31)))
+  (let* ((big-list (delete #f (map account->fq-cmd account-list)))
+         (cmd-list #f)
+         (currency-cmd-list (call-with-values 
+                             (lambda () (partition!
+                                         (lambda (cmd)
+                                           (not (eq? (car cmd) 'currency)))
+                                         big-list))
+                             (lambda (a b) (set! cmd-list a) b)))
+         (cmd-hash (make-hash-table 31)))
 
     ;; Now collect symbols going to the same backend.
     (item-list->hash! cmd-list cmd-hash car cdr hashq-ref hashq-set! #t)
+
     ;; Now translate to just what finance-quote-helper expects.
-    (hash-fold
-     (lambda (key value prior-result)
-       (cons (cons (fq-method-sym->str key) value)
-	     prior-result))
-     '()
-     cmd-hash)))
+    (append
+     (hash-fold
+      (lambda (key value prior-result)
+        (cons (cons (fq-method-sym->str key) value)
+              prior-result))
+      '()
+      cmd-hash)
+     (map (lambda (cmd) (cons (fq-method-sym->str (car cmd)) (list (cdr cmd))))
+          currency-cmd-list))))
 
   (define (fq-call-data->fq-calls fq-call-data)
     ;; take an output element from accounts->fq-call-data and return a
